#ifndef GPXI4009_HEADER
#define GPXI4009_HEADER

/*-------------------------------------------------------------------------*/
/* Defines                                                                 */
/*-------------------------------------------------------------------------*/
#define K_DLL_VERSION   151     // Version 1.51

#define K_DEV_MAX        16
#define K_DECADE_MAX      2

// XILINX command structure
#define K_HEADER_XILINX_INSTRUCTION_GLOBAL      0    // start byte
#define K_HEADER_XILINX_CARD_NUMBER             1    // device (1 ... K_DEV_MAX)
#define K_HEADER_XILINX_PORT_NUMBER             2    // not used
#define K_HEADER_XILINX_INSTRUCTION             3    // command
#define K_HEADER_XILINX_FIRST_DATA              4    
#define K_HEADER_XILINX_LENGTH                  4     

#define K_READ                      0
#define K_WRITE                     1

#define K_HEADER_XILINX_START_BYTE  0x48

// XILINX commands
#define K_XILINX_CMD_PO_RESET       0x10
#define K_XILINX_CMD_EXT_CLOCK      0x11
#define K_XILINX_CMD_REG_WRITE      0x12

// Update Relay
#define K_UPDATE_MODE_DECADE_BOTH                0
#define K_UPDATE_MODE_DECADE_1                   1
#define K_UPDATE_MODE_DECADE_2                   2

#define K_DECADE_1                               1
#define K_DECADE_2                               2

/*-------------------------------------------------------------------------*/
/* Typedefs                                                                */
/*-------------------------------------------------------------------------*/
typedef struct
	{
	unsigned long Version;
	unsigned long DevCount;
	unsigned long SW_XILINX[K_DEV_MAX];
	unsigned long BusIndexes[K_DEV_MAX];
	unsigned long DevIndexes[K_DEV_MAX];
	unsigned long GeoAddress[K_DEV_MAX];
	unsigned long SerialNumber[K_DEV_MAX];
	unsigned long Revision[K_DEV_MAX];
	} GPxi4009_StructDriverInfo;

typedef struct
	{
	unsigned long Version;
	unsigned long DevCount;
	unsigned long SW_XILINX[K_DEV_MAX];
	unsigned long BusIndexes[K_DEV_MAX];
	unsigned long DevIndexes[K_DEV_MAX];
	unsigned long GeoAddress[K_DEV_MAX];
	unsigned long SerialNumber[K_DEV_MAX];
	unsigned long Revision[K_DEV_MAX];
	unsigned long HwType[K_DEV_MAX];
	unsigned long Reserved[K_DEV_MAX];
	} GPxi4009_StructDriverInfo_ext;

typedef struct
	{ 
	unsigned long Decade_1_Ohm;
	unsigned long Decade_10_Ohm;
	unsigned long Decade_100_Ohm;
	unsigned long Decade_1k_Ohm;
	unsigned long Decade_10k_Ohm;
	unsigned long Decade_100k_Ohm;
	unsigned long Decade_1M_Ohm;
	} GPxi4009_StructReadDecadeData;
  
typedef struct
	{
	unsigned long Mode;
	unsigned long Register;
	unsigned long Value;
	} GPxi4009_StructXilinxReadWrite;

/*-------------------------------------------------------------------------*/
/* Errorcodes                                                              */
/*-------------------------------------------------------------------------*/
// 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|C|R|     Facility          |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  Sev:      is the severity code
//               00 - Success
//               01 - Informational
//               10 - Warning
//               11 - Error
//  C:        is the Customer code flag
//  R:        is a reserved bit
//  Facility: is the facility code
//  Code:     is the facility's status code

// error codes (all sources)
#define K_NO_ERROR                               0x00000000
// error driver
#define K_UNKNOWN_IOCTRL                         0xE0000001
#define K_NOTSUPPORTED_DEVICE                    0xE0000002
#define K_XILINX_DOWNLOADERROR                   0xE0000003
#define K_XILINX_WRITE_DATA_HEADER               0xE0000004

#define K_STATUS_EEPROM_NO_ACK                   0xE000000A
#define K_STATUS_EEPROM_NO_START_CONDITION       0xE000000B
#define K_STATUS_EEPROM_UNDEFINED                0xE000000C

#define K_UNDEFINED_ERROR                        0xE00000F0
// error library
#define K_INVALID_HANDLE                         0xE0000100
#define K_ERR_DLINVALIDID                        0xE0000101
#define K_ERR_DLINVALIDVERSION                   0xE0000102
#define K_ERR_DLSIZE                             0xE0000103
#define K_ERR_DLFILE                             0xE0000104
#define K_NO_CARD_PRESENT                        0xE0000105
#define K_DECADE_SELECTION_INVALID               0xE0000106


/******************************************
 * Goepel function
 *****************************************/
#ifdef __EXPORT_DLL	// creating DLL
	#define INTEGRATION_MODE	__declspec(dllexport)
#else				// using in customer application
	#define INTEGRATION_MODE	__declspec(dllimport)
#endif

#ifdef __cplusplus
	extern "C" {
#endif

INTEGRATION_MODE	int GPxi4009_GetDriverInfo(
													GPxi4009_StructDriverInfo	*pDriverInfo);
INTEGRATION_MODE	int GPxi4009_GetDriverInfo_ext(
													GPxi4009_StructDriverInfo_ext *pDriverInfo_ext);
INTEGRATION_MODE	int GPxi4009_DLL_Version(
													unsigned long	*pVersion);
INTEGRATION_MODE	int GPxi4009_XilinxDownload(
													unsigned long	Device,
													char			*Path);
INTEGRATION_MODE	int GPxi4009_XilinxWriteData(
													unsigned char	*Data,
													unsigned long	*Length);
INTEGRATION_MODE	int GPxi4009_UpdateDecade(
													unsigned long	Device,
													unsigned long	UpdateMode);
INTEGRATION_MODE	int GPxi4009_ReadDecadeData(
													unsigned long					Device,
													unsigned long					Decade,
													GPxi4009_StructReadDecadeData	*DecadeValue,
													double							*DecadeValueSingle);
INTEGRATION_MODE	int GPxi4009_WriteDecadeData(
													unsigned long	Device,
													unsigned long	Decade,
													double			DecadeValue);
INTEGRATION_MODE	int GPxi4009_ResetDecade(
													unsigned long	Device,
													unsigned long	ResetMode);
INTEGRATION_MODE	int GPxi4009_ReadWriteSpezial(
													unsigned char	*Data,
													unsigned long	*Length);

#ifdef __cplusplus
	}
#endif

#endif
